@@ -13,7 +13,6 @@
 mod ansi;
 mod generic;
 mod hive;
-pub mod keywords;
 mod mssql;
 mod mysql;
 mod postgresql;
@@ -31,6 +30,7 @@ pub use self::mysql::MySqlDialect;
 pub use self::postgresql::PostgreSqlDialect;
 pub use self::snowflake::SnowflakeDialect;
 pub use self::sqlite::SQLiteDialect;
+pub use crate::keywords;
 
 /// `dialect_of!(parser is SQLiteDialect |  GenericDialect)` evaluates
 /// to `true` iff `parser.dialect` is one of the `Dialect`s specified.
@@ -87,9 +87,8 @@ mod tests {
 
         assert!(dialect_of!(generic_holder is GenericDialect |  AnsiDialect),);
         assert!(!dialect_of!(generic_holder is  AnsiDialect));
-
-        assert!(dialect_of!(ansi_holder is  AnsiDialect));
-        assert!(dialect_of!(ansi_holder is  GenericDialect | AnsiDialect),);
-        assert!(!dialect_of!(ansi_holder is  GenericDialect | MsSqlDialect),);
+        assert!(dialect_of!(ansi_holder is AnsiDialect));
+        assert!(dialect_of!(ansi_holder is GenericDialect | AnsiDialect));
+        assert!(!dialect_of!(ansi_holder is GenericDialect | MsSqlDialect));
     }
 }
