@@ -114,6 +114,7 @@ define_keywords!(
     CEIL,
     CEILING,
     CHAIN,
+    CHANGE,
     CHAR,
     CHARACTER,
     CHARACTER_LENGTH,
@@ -181,6 +182,7 @@ define_keywords!(
     DISTRIBUTE,
     DOUBLE,
     DROP,
+    DUPLICATE,
     DYNAMIC,
     EACH,
     ELEMENT,
@@ -225,6 +227,7 @@ define_keywords!(
     GET,
     GLOBAL,
     GRANT,
+    GRANTED,
     GROUP,
     GROUPING,
     GROUPS,
@@ -318,6 +321,7 @@ define_keywords!(
     ON,
     ONLY,
     OPEN,
+    OPTION,
     OR,
     ORC,
     ORDER,
@@ -348,6 +352,7 @@ define_keywords!(
     PRECISION,
     PREPARE,
     PRIMARY,
+    PRIVILEGES,
     PROCEDURE,
     PURGE,
     RANGE,
@@ -395,12 +400,15 @@ define_keywords!(
     SECOND,
     SELECT,
     SENSITIVE,
+    SEQUENCE,
     SEQUENCEFILE,
+    SEQUENCES,
     SERDE,
     SERIALIZABLE,
     SESSION,
     SESSION_USER,
     SET,
+    SETS,
     SHOW,
     SIMILAR,
     SMALLINT,
@@ -432,6 +440,7 @@ define_keywords!(
     SYSTEM_TIME,
     SYSTEM_USER,
     TABLE,
+    TABLES,
     TABLESAMPLE,
     TBLPROPERTIES,
     TEMP,
@@ -468,6 +477,7 @@ define_keywords!(
     UNNEST,
     UPDATE,
     UPPER,
+    USAGE,
     USER,
     USING,
     UUID,
@@ -534,6 +544,7 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     Keyword::DISTRIBUTE,
     // for MSSQL-specific OUTER APPLY (seems reserved in most dialects)
     Keyword::OUTER,
+    Keyword::SET,
 ];
 
 /// Can't be used as a column alias, so that `SELECT <expr> alias`
